//
//  CapFileSupport.h
//  CapFileIO
//
//  Created by Neil Mackenzie on 15/12/2012.
//  Copyright (c) 2012 Capilano.com. All rights reserved.
//

#ifndef CapFileIO_CapFileSupport_h
#define CapFileIO_CapFileSupport_h

// Created  By Neil MacKenzie - Nov 2001.


#include <Carbon/Carbon.h>
#include "CapFiles.h"

#ifdef __cplusplus
extern "C" {
#endif
    
    extern    const int kCDSaveConst ;
    extern    const int kCDDontSaveConst ;
    extern    const int kCDCancelConst ;
    
    
    /*
     
     CreateShortFileName
     
     Given a name(most likely a filename) shorten it by taking "startingnumber" characters from the beginning of "name"
     and then put out "..."  followed by the last "endingNumber" of characters from name
     
     if startingnumber & endingNumber are zero or if  startingnumber+endingNumber  is greater "shortNameSize" then "shortName" is uneffected
     
     Input:
     name			-	file name to shorten
     startingnumber	-	number of characters to copy from start of "name"
     endingNumber	-	number of characters to copy from the end of "name"
     shortNameSize	-	length of shortName buffer
     
     Output
     shortName		-	shortened name
     
     Return			-  nothing
     
     */
    
    void CreateShortFileName(CFStringRef name, CFIndex startingNumber, CFIndex endingNumber, short shortenIfMoreThen , CFStringRef *shortName);
	
    /*
     
     CreateUniqueFileName
     
     Given two file paths , both ending with the same file name.  Genertate a unique name by
     looking where the paths first diverge. Use that directory to create a unique file name by combining
     the directory with the file name .
     
     Input:
     path1CF		-   complete path to "fileName"
     path2CF		-   complete path (but different) to "fileName"
     fileName		-   name of file
     
     
     Output:
     uniqueName		- uniqued name created by adding the diverging directory name to "fileName",  "../" separates the two. Caller has to release
     diffDirectroy  - the name of the directory where the two paths first diverg, looking from right to left in the path.  Caller has to release
     returns -		- true if uniqueName is created , false othewise
     
     Note-  You can pass in the complete path for both path1CF & path2CF or just the path the parent directory of fileName
     
     */
    
    Boolean  CreateUniqueFileName(CFStringRef path1CF, CFStringRef path2CF,CFStringRef fileName, CFStringRef *uniqueName,  CFStringRef *diffDirectroy);
	
	
	
    /*
     
     ShortenFileName
     
     Given a name(most likely a filename) short it, by taking "startingnumber" characters from the beginning of "name"
     and then put out ".."  followed by the last "endingNumber" of characters from name
     
     if startingnumber & endingNumber are zero then "shortName" is uneffected
     
     Input:
     name			-	file name to shorten
     nameLength		-	length of name buffer
     startingnumber	-	number of characters to copy from start of "name"
     endingNumber	-	number of characters to copy from the end of "name"
     shortenIfMoreThen - if lenght is less or equal to this then just copy across untouched.
     shortNameSize	-	length of shortName buffer
     
     Output
     shortName		- shortened name
     
     Return -  nothing
     
     */
    
    void ShortenFileName(char * name, short nameBufSize, short startingNumber, short endingNumber, short shortenIfMoreThen , char *shortName,short shortNameSize);
	
	
    CFStringRef CreateAbsolutePOSIXPath( FSRefPtr baseDirFSRef, CFStringRef thePath);
	
    Boolean CreateFSRefAndFileNameFromPath( FSRefPtr baseDirectoryRef, CFStringRef filePath, FSRefPtr containingDirRef, CFStringRef *fileNameCF);
	
    /*
     Given a root directory and a path string return an FSRef
     
	 Input:
	 rootWDRef  -  root directory
	 pathCF     -  path or just a file name
	 
	 Output:	The FSRef computed from above
	 
	 Note - A CFString version of GetWDRef
     */
    
    
    FSRef GetFSDirFromRootPath(FSRefPtr rootWDRef, CFStringRef pathCF);
    
    /*
     ConvertRelativeHFSToRelativePOSIX  --- converts "thePath" from HFS to POSIX path
     
     input -> thePath  -- in HFS style
     output -> thePath -- now in POSIX style
     */
	
	
    void ConvertRelativeHFSToRelativePOSIX( CFMutableStringRef thePath);
	
    /*
     GetWDRef  returns the working directory file reference
     given the pathname for a volume or folder.
     
     rootWD  is used as the root working directory for relative pathnames.
     */
    
    
    FSRef GetFSDirFromRootCPath( FSRef *rootWDRef, char folderName[] );
	
    
	
    char *PathNameForContainingDirectory( FSRef *parentDirRef, CFURLPathStyle thePathStyle, char *path, short sizeOfPathBuffer);
    
    
	
    /* Converts the given FSRef(still call WD for historic reasons)
     to a text pathname.  It is the caller's responsibility
     to deallocate the pathname string
     */
	
    Boolean CreatePathFromFSRef(FSRefPtr workingFSRef,CFURLPathStyle theStyle, char **valuePtr, short *valueLen);
    
    /* CFString version of CreatePathFromFSRef.  caller must release "thePathCF"
     */
    
    Boolean CreateCFPathFromFSRef(FSRefPtr workingFSRef,CFURLPathStyle theStyle, CFStringRef *thePathCF);
    
    /*----------------------------------------------------------------------*/
    /*
     Build an FSRef given a C style string
     
     In:
     path - complete path
     
     Out:
     fileFSRef - the created FSRef
     isDirectory -  whether it is a directory
     */
	
    
    // Create an FSRef given parent directory and file name
	
    OSErr CapMakeFSRefFromCStringParent(FSRef *parentDir, char *cStringName, FSRef *fileFSRef );
    OSErr CapMakeFSRefFromCString(char *path, FSRef *fileFSRef, Boolean *isDirectory );
    
    OSErr	CapMakeFSRefFromCFString(CFStringRef pathRef, FSRef *fileFSRef, Boolean *isDirectory );
    
    // Three routines to return the file name from an FSRef
	
    OSErr CapFSRefGetNameAsUniCode( const FSRef *fsRef, HFSUniStr255 *name );
    OSErr CapRefCreateNameAsCFString( const FSRef *fsRef, CFStringRef *name );
    OSErr CapFSRefGetNameAsCString( const FSRef *fsRef, char *stringPtr,  long strLen );
    
	
    // Create a file given a parent directory and a file name(C string and CFString versions)
    OSErr  CapFSCreateFileCFString(const FSRef *parentRef, CFStringRef fileName, FSCatalogInfoBitmap whichInfo, const FSCatalogInfo *catalogInfo, FSRef *newRef); 
    OSErr  CapFSCreateFile(const FSRef *parentRef, char* fileName, FSCatalogInfoBitmap whichInfo, const FSCatalogInfo *catalogInfo, FSRef *newRef) ;
    
#ifdef __cplusplus
}
#endif


#endif
